/* 
 * Copyright (C) 1996 Szeredi Miklos
 * Email: mszeredi@eik.bme.hu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "spperif.h"
#include "z80.h"

#include "snapshot.h"
#include "compr.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>

#define COMPRESS_SAVE 1

static FILE *savfp;
static int memptr;

int compr_read_byte()
{
  if(memptr < 0x10000) return z80_proc.mem[memptr++];
  else return -1;
}

void compr_put_byte(int i)
{
  putc(i, savfp);
}



void snsh_save(FILE *fp)
{
  int i;

  int to_comp = COMPRESS_SAVE;

  putc(RA, fp);     /* A reg */
  putc(RF, fp);     /* F reg */
  putc(RC, fp);     /* C reg */
  putc(RB, fp);     /* B reg */
  putc(RL, fp);     /* L reg */
  putc(RH, fp);     /* H reg */
  putc(PCL, fp);  /* PC reg */
  putc(PCH, fp);  
  putc(SPL, fp);  /* SP reg */
  putc(SPH, fp);
  putc(RI, fp);     /* I reg */
  putc(RR, fp);     /* R reg */
  
  putc(((RR >> 7) & 0x01) | 
       (((z80_proc.ula_outport & 0x07) << 1) & 0x0E) | 
       (to_comp ? 0x20 : 0), fp);
/*
                        Bit 0  : Bit 7 of the R-register
                        Bit 1-3: Border colour
                        Bit 4  : 1=Basic SamRom switched in
                        Bit 5  : 1=Block of data is compressed
                        Bit 6-7: No meaning
*/

  putc(RE, fp);    /* E reg */
  putc(RD, fp);    /* D reg */
  putc(CBK, fp);    /* C' reg */
  putc(BBK, fp);    /* B' reg */
  putc(EBK, fp);    /* E' reg */
  putc(DBK, fp);    /* D' reg */
  putc(LBK, fp);    /* L' reg */
  putc(HBK, fp);    /* H' reg */
  putc(ABK, fp);    /* A' reg */
  putc(FBK, fp);    /* F' reg */
  putc(YL, fp);    /* IY reg */
  putc(YH, fp);    
  putc(XL, fp);    /* IX reg */
  putc(XH, fp);

  putc(z80_proc.iff1, fp);         /* iff1 */
  putc(z80_proc.iff2, fp);         /* iff2 */
  
  putc(z80_proc.it_mode, fp); 

/*
                        Bit 0-1: Interrupt mode (0, 1 or 2)
                        Bit 2  : 1=Issue 2 emulation
                        Bit 3  : 1=Double interrupt frequency
                        Bit 4-5: 1=High video synchronisation
                                 3=Low video synchronisation
                                 0,2=Normal
                        Bit 6-7: 0=Cursor/Protek/AGF joystick
                                 1=Kempston joystick
                                 2=Sinclair 1 joystick
                                 3=Sinclair 2 joystick
*/

  if(!to_comp) 
    for(i = 16384; i < 65536; i++) putc(z80_proc.mem[i], fp);
  else {
    memptr = 0x4000;
    savfp = fp;
    compr();
  }
}


#define mygetc(x, fp) x = getc(fp)

void snsh_load(FILE *fp)
{
  int i;
  int tmpc;
  int compressed;

  mygetc(RA, fp);     /* A reg */
  mygetc(RF, fp);     /* F reg */
  mygetc(RC, fp);     /* C reg */
  mygetc(RB, fp);     /* B reg */
  mygetc(RL, fp);     /* L reg */
  mygetc(RH, fp);     /* H reg */
  mygetc(PCL, fp);  /* PC reg */
  mygetc(PCH, fp);  
  mygetc(SPL, fp);  /* SP reg */
  mygetc(SPH, fp);
  mygetc(RI, fp);     /* I reg */
  mygetc(RR, fp);     /* R reg */


  
  mygetc(tmpc, fp);

  RR = (RR & 0x7F) | ((tmpc << 7) & 0x80);
  z80_proc.ula_outport = (z80_proc.ula_outport & ~(0x07)) | 
    ((tmpc >> 1) & 0x07);

  compressed = 0;
  if(tmpc & 0x20) compressed = 1;

/*
                        Bit 0  : Bit 7 of the R-register
                        Bit 1-3: Border colour
                        Bit 4  : 1=Basic SamRom switched in
                        Bit 5  : 1=Block of data is compressed
                        Bit 6-7: No meaning
*/

  mygetc(RE, fp);    /* E reg */
  mygetc(RD, fp);    /* D reg */
  mygetc(CBK, fp);    /* C' reg */
  mygetc(BBK, fp);    /* B' reg */
  mygetc(EBK, fp);    /* E' reg */
  mygetc(DBK, fp);    /* D' reg */
  mygetc(LBK, fp);    /* L' reg */
  mygetc(HBK, fp);    /* H' reg */
  mygetc(ABK, fp);    /* A' reg */
  mygetc(FBK, fp);    /* F' reg */
  mygetc(YL, fp);    /* IY reg */
  mygetc(YH, fp);    
  mygetc(XL, fp);    /* IX reg */
  mygetc(XH, fp);

  mygetc(tmpc, fp);         /* iff1 */
  z80_proc.iff1 = tmpc ? 1 : 0;

  mygetc(tmpc, fp);         /* iff2 */
  z80_proc.iff2 = tmpc ? 1 : 0;
  
  mygetc(tmpc, fp); 

  z80_proc.it_mode = tmpc & 0x03;

/*
                        Bit 0-1: Interrupt mode (0, 1 or 2)
                        Bit 2  : 1=Issue 2 emulation
                        Bit 3  : 1=Double interrupt frequency
                        Bit 4-5: 1=High video synchronisation
                                 3=Low video synchronisation
                                 0,2=Normal
                        Bit 6-7: 0=Cursor/Protek/AGF joystick
                                 1=Kempston joystick
                                 2=Sinclair 1 joystick
                                 3=Sinclair 2 joystick
*/

  z80_proc.haltstate = 0;

  if(compressed) {
    int j, times, last_ed, ch;

    i = 16384;
    last_ed = 0;
    while(i < 65536) {
      mygetc(ch, fp);
      if(ch == 0xED) {
	if(last_ed) {
	  last_ed = 0;
	  mygetc(times, fp);
	  if(times == 0) {
	    fprintf(stderr, 
		    "Warning: illegal snapshot format, repetition is zero!\n");
 	    break; 
	  }
	  mygetc(ch, fp);
	  if(i + times > 65536) {
	    fprintf(stderr, 
		 "Warning: illegal snapshot format, too many repetitions\n");
	    times = 65536 - i;
	  }
	  for(j = 0; j < times; j++) z80_proc.mem[i++] = ch;
	}
	else last_ed = 1;
      }
      else if(last_ed) {
	z80_proc.mem[i++] = 0xED; 
	z80_proc.mem[i++] = ch;
	last_ed = 0;
      }
      else z80_proc.mem[i++] = ch;
    }
    if(getc(fp) != 0 || getc(fp) != 0xED || getc(fp) != 0xED || getc(fp) != 0)
      fprintf(stderr, "Warning: Illegal ending at %i (in file %li)\n", 
	      i, ftell(fp));
  }
  else for(i = 16384; i < 65536; i++) mygetc(z80_proc.mem[i], fp);

  sp_init_screen_mark();
}


#define MAXFILENAME 1024

static char *get_filename(void)
{
  static char buf[MAXFILENAME];
  char *name, *s;
  
  fgets(buf, MAXFILENAME, stdin);
  buf[MAXFILENAME-1] = '\0';

  s = buf;
  for(; *s && isspace(*s); s++);
  name = s;
  for(; *s && isgraph(*s); s++);
  *s = '\0';

  if(name == s) {
    printf("Canceled!\n");
    return NULL;
  }

  return name;
}

static char *default_ext(const char *name)
{
  static char buf[MAXFILENAME];
  char *extens;
  
  strncpy(buf, name, MAXFILENAME-10);
  buf[MAXFILENAME-10] = '\0';

  extens = buf+strlen(buf)-4;
  if(strlen(buf) < 4 || 
     (strcmp(extens, ".z80") != 0 &&
      strcmp(extens, ".Z80") != 0)) {
    if(buf[0] > 64 && buf[0] < 96)
      strcat(buf, ".Z80");
    else 
      strcat(buf, ".z80");
  }
  return buf;
}


void save_snapshot()
{
  char *name, *snsh_name;
  FILE *snsh;
  
  printf("Enter name of snapshot file to save:\n");

  name = get_filename();
  if(name == NULL) return;

  snsh_name = default_ext(name);

  snsh = fopen(snsh_name, "wb");
  if(snsh == NULL) {
    printf("Could not open snapshot file `%s', %s\n", 
	   snsh_name, strerror(errno));
    return;
  }

  snsh_save(snsh);
  fclose(snsh);

  printf("Saved snapshot to file %s\n", snsh_name);
}

void load_snapshot_file(char *name)
{
  FILE *snsh;
  char *snsh_name;

  snsh_name = default_ext(name);

  snsh = fopen(snsh_name, "rb");
  if(snsh == NULL) {
    printf("Could not open snapshot file `%s', %s\n", 
	   snsh_name, strerror(errno));
    return;
  }

  snsh_load(snsh);

  fclose(snsh);
}

void load_snapshot()
{
  char *name;
  
  printf("Enter name of snapshot file to load:\n");

  name = get_filename();
  if(name == NULL) return;

  load_snapshot_file(name);
}

